{% load i18n %}

<p>{% blocktrans %}Are you sure you want to delete {{ object }}{% endblocktrans %}</p>

<form id="delete_form" action="{{ view.delete_url }}" method="POST">
  {% csrf_token %}

  <div class="btn-group form-controls">
    <button type="submit" class="btn btn-primary protected">
      {% trans "Delete" %}
    </button>
    <a href="#" class="btn cancel">{% trans "Cancel" %}</a>
  </div>
</form>
