{% load i18n %}

<div class="modal fade" id="{% block id %}{% endblock %}">

  <div class="modal-header">
    {% block header %}
    <a class="close" data-dismiss="modal">&times; {% trans "Close window" %}</a>
    <h3>{% block title %}{% endblock %}</h3>
    {% endblock %}
  </div>

  <div class="modal-body">
    {% block body %}{% endblock %}
  </div>

  <div class="modal-footer">
    {% block footer %}{% endblock %}
  </div>
  
</div>
